/*
								+---------------------------------+
								|                                 |
								| *** Pack progress indicator *** |
								|                                 |
								|  Copyright   -tHE SWINe- 2008  |
								|                                 |
								|          Progress.cpp           |
								|                                 |
								+---------------------------------+
*/

#include <vector>
#include "../berLame_src/CallStack.h"
#include <stdio.h>
#include <algorithm>
#include "../berLame_src/Timer.h"
#include "../berLame_src/Dir.h"
#include "../berLame_src/Compress.h"
#include "../berLame_src/LamePak2.h"
#include "Progress.h"

/*
 *								=== CMyProgressIndicator ===
 */

/*
 *	CMyProgressIndicator::CMyProgressIndicator(uint64_t n_total_size, bool b_size_valid,
 *		CTimer &r_timer, float f_start_time)
 *		- default constructor; sets total amount of data to be packed, wheter it is valid,
 *		  timer and time when the packer was started
 */
CMyProgressIndicator::CMyProgressIndicator(uint64_t n_total_size, bool b_size_valid,
	CTimer &r_timer, float f_start_time)
	:m_r_timer(r_timer), m_f_start_time(f_start_time),
	m_n_total_size(n_total_size), m_b_size_valid(b_size_valid),
	m_n_total_in(0), m_n_total_out(0), m_n_block(0)
{}

/*
 *	virtual void CMyProgressIndicator::On_TOCCompress(unsigned int n_in, unsigned int n_out)
 *		- called when table of contents is compressed
 *		- n_in is amount of data to be packed and n_out is resulting packed size
 */
void CMyProgressIndicator::On_TOCCompress(unsigned int n_in, unsigned int n_out)
{
	if(n_out < n_in) {
		printf("toc takes up: ");
		PrintSize(n_out);
		printf(" (orig: ");
		PrintSize(n_in);
		printf(")\n");
	} else {
		printf("toc takes up: ");
		PrintSize(n_out);
		printf(" (raw)\n");
	}
	// print verbose info

	m_n_total_in += n_in;
	m_n_total_out += n_out;
	// sum up total file sizes

	++ m_n_block;
	// increment block counter
}

/*
 *	virtual bool CMyProgressIndicator::On_BlockCompress(unsigned int n_in, unsigned int n_out)
 *		- called when data block is compressed
 *		- n_in is amount of data to be packed and n_out is resulting packed size
 *		- returning false in this function makes packer fail immediately
 */
bool CMyProgressIndicator::On_BlockCompress(unsigned int n_in, unsigned int n_out)
{
	
	float f_time = m_r_timer.f_Time() - m_f_start_time;
	// make timer tick so it doesn't overflow

	if(m_b_size_valid) {
		_ASSERTE(m_n_total_in <= UINT64_MAX - n_in);
		_ASSERTE(m_n_total_out <= UINT64_MAX - n_out);
		m_n_total_in += n_in;
		m_n_total_out += n_out;
		// sum up total file sizes (shouldn't overflow if m_b_size_valid is true)

		float f_ratio = float(n_out) / n_in;
		// calc current block compression ratio

		float f_estimated_time = f_time / ((m_n_total_size <= INT64_MAX)?
			float(int64_t(m_n_total_in)) / int64_t(m_n_total_size) :
			float(int64_t(m_n_total_in / 2)) / int64_t(m_n_total_size / 2));
		if(f_estimated_time < f_time)
			f_estimated_time = f_time;
		// calc remaining time

		printf("writing block %d: ", m_n_block);
		PrintSize(n_out);
		if(n_in != n_out)
			printf(", ratio: %.3f, time: ", f_ratio);
		else
			printf(", ratio: (raw), time: ");
		PrintTime(f_time);
		printf(", est: ");
		PrintTime(f_estimated_time);
		printf(" \r");
	} else {
		printf("writing block %d: ", m_n_block);
		PrintSize(n_out);
		printf(", time: ");
		PrintTime(f_time);
		printf(" \r");
	}
	// print verbose info

	++ m_n_block;
	// increment block counter

	return true; // don't want to interrupt packing
}

/*
 *	void CMyProgressIndicator::PrintCompression()
 *		- prints final average ratio and difference size
 */
void CMyProgressIndicator::PrintCompression()
{
	if(!m_b_size_valid)
		return;
	// can't tell

	float f_ratio = (m_n_total_in <= INT64_MAX && m_n_total_out <= INT64_MAX)?
		float(int64_t(m_n_total_out)) / int64_t(m_n_total_in) :
		float(int64_t(m_n_total_out / 2)) / int64_t(m_n_total_in / 2);
	// calc average compression ratio

	int64_t n_saved = m_n_total_size - m_n_total_out;
	// how much did we save by compression (can be negative as well)

	printf("avg ratio: %.3f, pack difference: ", f_ratio);
	if(n_saved >= 0)
		PrintSize(n_saved);
	else {
		printf("+");
		PrintSize(-n_saved);
	}
	printf("\n");
	// prints final average ratio and difference size
}

/*
 *	static void CMyProgressIndicator::PrintTime(float f_time)
 *		- prints time f_time [seconds] as hours:minutes:seconds.seconds/100
 *		  to stdout (no newlines)
 */
void CMyProgressIndicator::PrintTime(float f_time)
{
	float f_seconds = f_time;
	int n_minutes = int(f_seconds / 60);
	f_seconds -= n_minutes * 60;
	int n_hours = n_minutes / 60;
	n_minutes -= n_hours * 60;
	// calc human-readable time

	printf("%02d:%02d:%05.2f", n_hours, n_minutes, f_seconds);
}

/*
 *	void CMyProgressIndicator::PrintSize(uint64_t n_size)
 *		- prints size n_size with proper size unit (B / kB / MB / GB / TB)
 */
void CMyProgressIndicator::PrintSize(uint64_t n_size)
{
	int n_unit = 0;
	double f_size;
	if(n_size < INT64_MAX)
		f_size = double(int64_t(n_size));
	else {
		f_size = double(int64_t(n_size / 1024));
		n_unit = 1;
	}
	_ASSERTE(f_size >= 0);
	// convert total size to double

	for(; n_unit < 4 && f_size >= 1024 ; ++ n_unit)
		f_size /= 1024;
	// choose appropriate unit

	const char *p_s_unit[] = {"B", "k", "M", "G", "T"};
	if(n_unit)
		printf("%.2f%s", f_size, p_s_unit[n_unit]);
	else
		printf("%d%s", int(f_size), p_s_unit[n_unit]);
}

/*
 *								=== ~CMyProgressIndicator ===
 */
